"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.trace = exports.getTracerInfo = exports.addInstrumentation = exports.LUMIGO_ENDPOINT = void 0;
const api_1 = require("@opentelemetry/api");
const LumigoHttpInstrumentation_1 = __importDefault(require("./instrumentros/LumigoHttpInstrumentation"));
const LumigoExpressInstrumentation_1 = __importDefault(require("./instrumentros/LumigoExpressInstrumentation"));
const sdk_trace_node_1 = require("@opentelemetry/sdk-trace-node");
const sdk_trace_base_1 = require("@opentelemetry/sdk-trace-base");
const instrumentation_1 = require("@opentelemetry/instrumentation");
const resources_1 = require("@opentelemetry/resources");
const exporter_trace_otlp_http_1 = require("@opentelemetry/exporter-trace-otlp-http");
const utils_1 = require("./utils");
const logLevel = (process.env.LUMIGO_DEBUG || 'false').toLowerCase() === 'true'
    ? api_1.DiagLogLevel.ALL
    : api_1.DiagLogLevel.ERROR;
api_1.diag.setLogger(new api_1.DiagConsoleLogger(), logLevel);
exports.LUMIGO_ENDPOINT = 'http://lumigo-wrapper-collector.golumigo.com:55681/v1/trace' || process.env.LUMIGO_ENDPOINT;
let isTraced = false;
const MODULES_TO_INSTRUMENT = ['express', 'http', 'https'];
const externalInstrumentations = [];
const addInstrumentation = (instrumentation) => {
    if (isTraced) {
        console.warn(`Lumigo already traced, Try to first add your instrumentation and then calling trace()`);
    }
    else if (instrumentation instanceof instrumentation_1.InstrumentationBase) {
        externalInstrumentations.push(instrumentation);
    }
    else {
        console.warn(`instrumentation is not instance of InstrumentationBase`);
    }
};
exports.addInstrumentation = addInstrumentation;
const safeRequire = (libId) => {
    try {
        const customReq = 
        // eslint-disable-next-line no-undef,camelcase
        // @ts-ignore
        typeof __non_webpack_require__ !== 'undefined' ? __non_webpack_require__ : require;
        return customReq(libId);
    }
    catch (e) {
        try {
            const customReq = 
            // eslint-disable-next-line no-undef,camelcase
            // @ts-ignore
            typeof __non_webpack_require__ !== 'undefined' ? __non_webpack_require__ : require;
            const path = customReq.resolve(libId, {
                paths: [...process.env.NODE_PATH.split(':'), '/var/task/node_modules/'],
            });
            return customReq(path);
        }
        catch (e) {
            if (e.code !== 'MODULE_NOT_FOUND') {
                api_1.diag.warn('Cant load Module', {
                    error: e,
                    libId: libId,
                });
            }
        }
    }
    return undefined;
};
const getTracerInfo = () => {
    return (0, utils_1.safeExecute)(() => {
        // eslint-disable-next-line @typescript-eslint/no-var-requires
        const pkg = require('../package.json');
        const { name, version } = pkg;
        return { name, version };
    }, 'Failed to get wrapper version', 'warn', { name: '@lumigo/microservices-node-tracer', version: '0.0.0' })();
};
exports.getTracerInfo = getTracerInfo;
function requireIfAvailable(names) {
    names.forEach((name) => safeRequire(name));
}
const trace = (lumigoToken = '', serviceName = 'service-name', endpoint = exports.LUMIGO_ENDPOINT) => {
    try {
        if (process.env.LUMIGO_SWITCH_OFF && process.env.LUMIGO_SWITCH_OFF.toLowerCase() === 'true') {
            api_1.diag.debug('Lumigo is switched off');
            return;
        }
        if (isTraced) {
            api_1.diag.debug('Lumigo already traced');
            return;
        }
        const exporter = new exporter_trace_otlp_http_1.OTLPTraceExporter({
            // @ts-ignore
            serviceName,
            url: endpoint,
        });
        const config = {
            resource: new resources_1.Resource({
                lumigoToken: lumigoToken.trim(),
                'service.name': serviceName,
                runtime: `node${process.version}`,
                tracerVersion: (0, exports.getTracerInfo)().version,
                framework: 'express',
                envs: JSON.stringify(process.env),
            }),
        };
        const traceProvider = new sdk_trace_node_1.NodeTracerProvider(config);
        traceProvider.addSpanProcessor(new sdk_trace_base_1.BatchSpanProcessor(exporter));
        traceProvider.register();
        (0, instrumentation_1.registerInstrumentations)({
            instrumentations: [
                // @ts-ignore
                new LumigoHttpInstrumentation_1.default(lumigoToken, endpoint),
                // @ts-ignore
                new LumigoExpressInstrumentation_1.default(),
                ...externalInstrumentations,
            ],
        });
        requireIfAvailable([
            ...MODULES_TO_INSTRUMENT,
            ...JSON.parse(process.env.MODULES_TO_INSTRUMENT || '[]'),
        ]);
        isTraced = true;
        api_1.diag.debug('Lumigo instrumentation started');
    }
    catch (e) {
        console.error('Lumigo tracer had an Error: ', e);
    }
};
exports.trace = trace;
if (process.env.LUMIGO_TOKEN && process.env.LUMIGO_SERVICE_NAME && !isTraced) {
    (0, exports.trace)(process.env.LUMIGO_TOKEN, process.env.LUMIGO_SERVICE_NAME, process.env.LUMIGO_ENDPOINT || exports.LUMIGO_ENDPOINT);
}
module.exports = {
    trace: exports.trace,
    LumigoHttpInstrumentation: LumigoHttpInstrumentation_1.default,
    LumigoExpressInstrumentation: LumigoExpressInstrumentation_1.default,
    LumigoInstrumentations: (lumigoToken) => [
        new LumigoHttpInstrumentation_1.default(lumigoToken),
        new LumigoExpressInstrumentation_1.default(),
    ],
};
//# sourceMappingURL=wrapper.js.map